
#include <stdio.h>

#include "lion.h"


#include "version.h"




static int master_switch = 0;
static int server_port   = 2121;




int server_debug = 0;




void exit_interrupt(void)
{

	master_switch = 1;

}












int main(int argc, char **argv)
{

	arguments(argc, argv);


	printf("[SSL]-lbnc FTP Relay Server.\n");

	printf("Version %u.%u Protocol %u.%u build %u (%s@%s) - %s\n\n",
		   VERSION_MAJOR,
		   VERSION_MINOR,
		   PROTOCOL_MAJOR,
		   PROTOCOL_MINOR,
		   VERSION_BUILD,
		   VERSION_USER,
		   VERSION_HOST,
		   VERSION_DATE);
	

	// If they send us INTR or HUP, lets shut down cleanly.
	signal(SIGINT, exit_interrupt);
#ifndef WIN32
	signal(SIGHUP, exit_interrupt);
#endif



	printf("Initialising Network...\n");

	lion_init();

	printf("Network Initialised.\n");


#ifdef WITH_SSL

	printf("Initialising SSL...\n");

	printf("SSL/TLS Initialised successfully. Server SSL %s, Client SSL %s\n",
		   server_SSL ? "enabled" : "disabled",
		   client_SSL ? "enabled" : "disabled");
	

#endif




	// Create an initial game
		
	printf("Initialising Socket...\n");


	
	printf("Running...\n");

	while( !master_switch ) {

		lion_poll(0, 10);     // This blocks. (by choice, FYI).

	}
	printf("\n");


	printf("Releasing Socket...\n");

	printf("Socket Released.\n");


	lion_free();

	printf("Network Released.\n");

	printf("Done\n");

	return 0; // Avoid warning
}







void arguments(int argc, char **argv)
{
	int opt;
	
	if (argc <= 1) {
		
		printf("%s - A(nother) FTP Relay Program.\n", argv[0]);
		printf("%s [options] host port\n\n", argv[0]);
		printf("host is best put in IP dot-notation.\n\n");
		printf("  options:\n");
		printf("  -p        : Only use PASV to server (Change clients PORT to PASV)\n");
		printf("  -o        : Only use PORT to server (Change clients PASV to PORT)\n");
		printf("  -d        : debug print\n");
		printf("  -l [port] : Set local listen port\n");
		printf("  -i [ip]   : Force use of 'ip' to reply to PASV to clients (for NATed hosts)\n");
		printf("  -a        : Send non-standard IDEN packet to FTPD\n");

		printf("  -t        : Enable internal 'SITE STaT' command to view internal state\n");
#ifdef WITH_SSL
		printf("\n");
		printf("  -s        : Disable SSL on lbnc <--> FTP server connection \n");
		printf("  -S        : Force   SSL on lbnc <--> FTP server connection \n");
		printf("  -c        : Disable SSL on lbnc <--> FTP client connection \n");
		printf("  -C        : Force   SSL on lbnc <--> FTP client connection \n");
		printf("\n");
		printf("  -v        : Force SSL on Server Data sessions OFF\n");
		printf("  -V        : Force SSL on Server Data sessions ON\n");
		printf("  -b        : Force SSL on Client Data sessions OFF\n");
		printf("  -B        : Force SSL on Client Data sessions ON\n");
		printf("\n");
		printf("  -e [path] : Specify egd socket for systems without /dev/random\n");
		printf("  -r [path] : Specify SSL PEM certificate file\n");
		printf("  -L [list] : Specify SSL ciphers wanted\n");


		printf("Note the default is to let client specify both SSL, and Data mode\n");
		printf("security level (using PROT <level>).\n");
		printf("\nTODO: So far, only client-side SSL functionality is implemented!\n");
#endif
		printf("\n");
		exit(0);
	}


	while ((opt=getopt(argc, argv, 
					   "pl:odi:at"
#ifdef WITH_SSL
					   "sScCvVbBe:r:L:"
#endif
					   
					   )) != -1) {

		switch(opt) {

		case 'p':
			server_onlypasv ^= 1;
			server_onlyport = !server_onlypasv;
			break;

		case 'o':
			server_onlyport ^= 1;
			server_onlypasv = !server_onlyport;
			break;

		case 'd':
			server_debug ^= 1;
			break;

		case 'a':
			server_sendident ^= 1;
			break;

		case 'l':
			server_port = atoi( optarg );
			break;

		case 'i':
			server_fakeipclient = ntohl(sockets_gethost(optarg));
			break;

		case 't':
			server_extrastat ^= 1;
			break;


#ifdef WITH_SSL
		case 's':
			server_SSL = 1; // 1 is disable
			break;
		case 'S':
			server_SSL = 2; // 2 is force on
			break;
		case 'c':
			client_SSL = 1; // 1 is disable
			break;
		case 'C':
			client_SSL = 2; // 2 is force on
			break;

		case 'v':
			server_SSL_data = 1; // 1 is disable
			break;
		case 'V':
			server_SSL_data = 2; // 2 is force on
			break;
		case 'b':
			client_SSL_data = 1; // 1 is disable
			break;
		case 'B':
			client_SSL_data = 2; // 2 is force on
			break;
		case 'e':
			lion_ssl_egdfile( optarg );
			break;
		case 'r':
			lion_ssl_rsafile( optarg );
			break;
		case 'L':
			lion_ssl_ciphers( optarg );
			break;
#endif

		default:
			printf("Unknown option.\n");
			break;
		}
	}

  argc -= optind;
  argv += optind;

  if (argc == 2) {

	  relay_host = argv[0];
	  relay_port = atoi(argv[1]);

  } else {

	  printf("Also need host and port\n");

	  exit(0);
  }


}
